package com.gic.evaluate.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.dto.EvaluateMsgLogDTO;
import com.gic.evaluate.entity.TabEvaluateMsgLog;
import com.gic.evaluate.entity.TabEvaluateOrderConfig;
import com.gic.evaluate.service.EvaluateMsgLogApiService;
import com.gic.evaluate.service.EvaluateMsgLogService;
import com.gic.evaluate.service.EvaluateOrderConfigService;
import com.gic.marketing.process.api.dto.notice.MQNoticeSendDTO;
import com.gic.marketing.process.api.service.notice.NoticeRouterApiService;
import com.gic.marketing.process.api.utils.NoticeConstant;
import com.gic.member.api.dto.McuBaseInfoDTO;
import com.gic.member.api.service.CuDetailService;
import com.gic.mq.sdk.GicMQClient;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2020/2/6 13:40
 */
@Service("evaluateMsgLogApiService")
public class EvaluateMsgLogApiServiceImpl implements EvaluateMsgLogApiService {

    private Logger logger = LogManager.getLogger(EvaluateMsgLogApiServiceImpl.class);

    @Autowired
    private EvaluateMsgLogService evaluateMsgLogService;
    @Autowired
    private EvaluateOrderConfigService evaluateOrderConfigService;
    @Autowired
    private CuDetailService cuDetailService;
    @Autowired
    private NoticeRouterApiService noticeRouterApiService;

    @Override
    public ServiceResponse<Void> saveEvaluateMsgLog(EvaluateMsgLogDTO evaluateMsgLogDTO) {
        logger.info("保存评价提醒日志信息：{}", JSON.toJSONString(evaluateMsgLogDTO));
        TabEvaluateMsgLog oldMsgLog = evaluateMsgLogService.getByOrderId(evaluateMsgLogDTO.getOrderId());
        if (oldMsgLog != null) {
            logger.info("该订单已存在, orderId:{}", evaluateMsgLogDTO.getOrderId());
            return EnterpriseServiceResponse.success();
        }
        TabEvaluateMsgLog evaluateMsgLog = EntityUtil.changeEntityByJSON(TabEvaluateMsgLog.class, evaluateMsgLogDTO);

        TabEvaluateOrderConfig evaluateOrderConfig = evaluateOrderConfigService.getEvaluateOrderConfig(evaluateMsgLogDTO.getEnterpriseId());
        Date now = new Date();

        if (evaluateOrderConfig.getMindHour() != null) {
            // 第一次提醒
            Date sendTime = DateUtil.stringToDate(DateUtil.dateToString(DateUtil.addNumForDay(now, evaluateOrderConfig.getMindHour())) + " 19:00:00");
            evaluateMsgLog.setFirstSendTime(sendTime);
            evaluateMsgLog.setFirstStatus(1);
        }
        if (evaluateOrderConfig.getRemindDay() != null) {
            // 第二次提醒
            Date sendTime = DateUtil.stringToDate(DateUtil.dateToString(DateUtil.addNumForDay(now, evaluateOrderConfig.getRemindDay())) + " 19:00:00");
            evaluateMsgLog.setSecondSendTime(sendTime);
            evaluateMsgLog.setSecondStatus(1);
        }
        if (Constants.OPEN.equals(evaluateOrderConfig.getReview()) && evaluateOrderConfig.getReviewRemind() != null) {
            // 追评提醒
            Date date = DateUtil.addNumForDay(DateUtil.addNumForDay(now, evaluateOrderConfig.getEvaluateLimitday()), evaluateOrderConfig.getReviewRemind());
            Date sendTime = DateUtil.stringToDate(DateUtil.dateToString(date) + " 19:00:00");
            evaluateMsgLog.setReviewSendTime(sendTime);
            evaluateMsgLog.setReviewStatus(1);
        }
        evaluateMsgLogService.save(evaluateMsgLog);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> timerSendMemberMessage() {
        // todo 查询到所有的企业id
        List<Integer> enterpriseIds = Arrays.asList(1129);
        for (Integer enterpriseId : enterpriseIds) {
            GicMQClient client = GICMQClientUtil.getClientInstance();
            try {
                client.sendMessage("enterSendMemberMessage", enterpriseId.toString());
            } catch (Exception e) {
                logger.info("发送队列错误", e);
            }
            //            sendMessageToMQ("quartzEnterSendMemberMessage", message);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> queueSendMemberMessage(String message) {
        Integer enterpriseId = Integer.valueOf(message);
//        List<TabEvaluateMsgLog> firstMsgLogs = this.evaluateMsgLogService.listFirstMsgLogs(enterpriseId);
//        sendMsg(enterpriseId, firstMsgLogs);
//        List<TabEvaluateMsgLog> secondMsgLogs = this.evaluateMsgLogService.listSecondMsgLogs(enterpriseId);
//        sendMsg(enterpriseId, secondMsgLogs);
//        List<TabEvaluateMsgLog> reviewMsgLogs = this.evaluateMsgLogService.listReviewMsgLogs(enterpriseId);
//        sendMsg(enterpriseId, reviewMsgLogs);


        return null;
    }

    private void sendMsg(Integer enterpriseId, List<TabEvaluateMsgLog> firstMsgLogs) {
        for (TabEvaluateMsgLog log : firstMsgLogs) {

            // 发送信息
            MQNoticeSendDTO notice = new MQNoticeSendDTO();

            // 企业ID
            notice.setEnterpriseId(enterpriseId);

            // 会员卡域id
            notice.setMemberAreaId(log.getMemberAreaId());

            // 事件code
            notice.setNoticeCode(NoticeConstant.NOTICE_ORDER_EVALUATE);

            // 会员ecuId
            McuBaseInfoDTO mcuDTO = cuDetailService.getMcuPopoutInfo(log.getMcuId(), log.getEnterpriseId()).getResult();
//            notice.setEcuId(mcuDTO.get);

            Map<String, String> valueMap = new HashMap<>();
            //订单编号
            valueMap.put(NoticeConstant.PARAMS_ORDER_NUMBER, log.getOrderNumber());
            //商品详情
            valueMap.put(NoticeConstant.PARAMS_GOODS_DETAIL, log.getGoodsInfo());
            //订单金额
            valueMap.put(NoticeConstant.PARAMS_ORDER_AMOUNT, log.getSaleAmount().toString());
            //消费时间（时间）
            valueMap.put(NoticeConstant.PARAMS_PAY_TIME, DateUtil.dateToStr(log.getReceiptsDate(), DateUtil.FORMAT_DATE_10));

            // 事件需要的事件级变量 ，具体变量参考附件
            notice.setValueMap(valueMap);

            this.noticeRouterApiService.putNoticeSendRouter(notice);
        }
    }
}
